/***************************************************************************
 *   Copyright (C) 2007 by Ken Werner                                      *
 *   ken.werner@web.de                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "nvidiathermalsrc.h"
#include <qstringlist.h>
#include <qregexp.h>
#include <klocale.h>

// waitpid
#include <sys/wait.h>

NVidiaThermalSrc::NVidiaThermalSrc(QWidget* inParent, const QString& inID, const QString& inName):
		LabelSource(inParent){
	mID = inID;
	mName = inName;
	mDescription = i18n("This source is provided by the nVidia GPU card driver tools");
	mProcess = new QProcess(this);
	mProcess->addArgument("nvidia-settings");
	mProcess->addArgument("-n");
	mProcess->addArgument("-q");
	mProcess->addArgument(mID);

	mRefreshTimer = new QTimer(this, "default refresh handler" );
	// connect the timer
	connect(mRefreshTimer, SIGNAL(timeout()), this, SLOT(fetchValue()));
	connect(mProcess, SIGNAL(processExited()), this, SLOT(evaluateStdout()));
	connect(this, SIGNAL(enabledChanged(bool, Source*)), this, SLOT(enable(bool)));
}

NVidiaThermalSrc::~NVidiaThermalSrc(){
	delete mProcess;
}

std::list<Source*>NVidiaThermalSrc::createInstances(QWidget* inParent){
	std::list<Source*> list;
	// see if the path contains nvidia-settings. if yes, 
	// execute it and look for the query type "GPUCoreTemp"
	// and "GPUAmbientTemp" and create two instances for each
	// of them
	QStringList cmd;
	cmd << "nvidia-settings";
	cmd << "-n"; // don't load config
	cmd << "-q" << "GPUCoreTemp";
	cmd << "-q" << "GPUAmbientTemp";
	
	// start nvidia-settings, if available and wait for it to exit.
	QProcess proc(cmd);
	if(!proc.start() || waitpid(proc.processIdentifier(), NULL, 0) < 0)
		return list;
	
	// now see what it printed...
	QString pout = proc.readStdout();
	if(pout.contains("Attribute 'GPUCoreTemp'"))
		list.push_back(new NVidiaThermalSrc(inParent, "GPUCoreTemp", "NVidiaCore"));
	if(pout.contains("Attribute 'GPUAmbientTemp'"))
		list.push_back(new NVidiaThermalSrc(inParent, "GPUAmbientTemp", "NVidiaAmbient"));
	
	return list;
}

void NVidiaThermalSrc::enable(bool inEnable){
	if(inEnable && !mRefreshTimer->isActive()){ // start the timer
		fetchValue();
		mRefreshTimer->start(3000);
	}else if(!inEnable && mRefreshTimer->isActive()) // stops the timer
		mRefreshTimer->stop();
}

void NVidiaThermalSrc::evaluateStdout(){
		QString val = i18n("n/a");
		QString pout = mProcess->readStdout();
		QRegExp regexp("Attribute\\s'" + mID + "'.*(\\d+)\\.");
		if(regexp.search(pout) != -1)
		    val = formatTemperature(regexp.cap(1));
		mValue = val;
		emit valueUpdated(mValue); // calls updateLabel(mValue) of LabelSource
}

QString NVidiaThermalSrc::fetchValue(){
	if(!mProcess->isRunning())
		if(!mProcess->start())
			mValue = i18n("n/a"); 
	return getValue();
}

